"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBpmEncoder = function (imgData) {
    var width = imgData.width;
    var height = imgData.height;
    var extraBytes = width % 4;
    var rgbSize = height * (3 * width + extraBytes);
    var headerInfoSize = 40;
    /******************header***********************/
    var flag = 'BM';
    var reserved = 0;
    var offset = 54;
    var fileSize = rgbSize + offset;
    var planes = 1;
    var bitPP = 24;
    var compress = 0;
    var hr = 0;
    var vr = 0;
    var colors = 0;
    var importantColors = 0;
    return function (buffer) {
        var tempBuffer = Buffer.alloc(offset + rgbSize);
        var pos = 0;
        tempBuffer.write(flag, pos, 2);
        pos += 2;
        tempBuffer.writeUInt32LE(fileSize, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(reserved, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(offset, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(headerInfoSize, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(width, pos);
        pos += 4;
        tempBuffer.writeInt32LE(-height, pos);
        pos += 4;
        tempBuffer.writeUInt16LE(planes, pos);
        pos += 2;
        tempBuffer.writeUInt16LE(bitPP, pos);
        pos += 2;
        tempBuffer.writeUInt32LE(compress, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(rgbSize, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(hr, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(vr, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(colors, pos);
        pos += 4;
        tempBuffer.writeUInt32LE(importantColors, pos);
        pos += 4;
        var i = 0;
        var rowBytes = 3 * width + extraBytes;
        for (var y = 0; y < height; y++) {
            for (var x = 0; x < width; x++) {
                var p = pos + y * rowBytes + x * 3;
                i++; // a
                tempBuffer[p] = buffer[i++]; // b
                tempBuffer[p + 1] = buffer[i++]; // g
                tempBuffer[p + 2] = buffer[i++]; // r
            }
            if (extraBytes > 0) {
                var fillOffset = pos + y * rowBytes + width * 3;
                tempBuffer.fill(0, fillOffset, fillOffset + extraBytes);
            }
        }
        return {
            shiftPos: pos,
            rowBytes: rowBytes,
            data: tempBuffer
        };
    };
};
//# sourceMappingURL=bpmEncoder.js.map