"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var bmp_js_1 = require("bmp-js");
var jimp_1 = __importDefault(require("jimp"));
exports.mixValues = function (value1, value1Opacity, value2) {
    return Math.round(value2 * (1 - value1Opacity) + value1 * value1Opacity);
};
exports.mixColors = function (_a) {
    var color = _a.color, colorOpacity = _a.colorOpacity, backgroundColor = _a.backgroundColor;
    return {
        red: exports.mixValues(color.red, colorOpacity, backgroundColor.red),
        green: exports.mixValues(color.green, colorOpacity, backgroundColor.green),
        blue: exports.mixValues(color.blue, colorOpacity, backgroundColor.blue),
    };
};
var getRectPixelColor = function (params) {
    var imageDstBuffer = params.imageDstBuffer, pixelIndex = params.pixelIndex, color = params.color, opacity = params.opacity;
    if (opacity === 1) {
        return color;
    }
    var backgroundPixelColor = {
        blue: imageDstBuffer.data[pixelIndex],
        green: imageDstBuffer.data[pixelIndex + 1],
        red: imageDstBuffer.data[pixelIndex + 2],
    };
    var resultColor = exports.mixColors({
        color: color,
        colorOpacity: opacity,
        backgroundColor: backgroundPixelColor,
    });
    return resultColor;
};
exports.drawRect = function (_a) {
    var imageDstBuffer = _a.imageDstBuffer, position = _a.position, size = _a.size, color = _a.color, opacity = _a.opacity;
    for (var currY = position.y; currY < position.y + size.height; currY++) {
        for (var currX = position.x; currX < position.x + size.width; currX++) {
            var pixelIndex = imageDstBuffer.shiftPos + currY * imageDstBuffer.rowBytes + currX * 3;
            var pixelColor = getRectPixelColor({
                imageDstBuffer: imageDstBuffer,
                pixelIndex: pixelIndex,
                color: color,
                opacity: opacity,
            });
            imageDstBuffer.data[pixelIndex] = pixelColor.blue;
            imageDstBuffer.data[pixelIndex + 1] = pixelColor.green;
            imageDstBuffer.data[pixelIndex + 2] = pixelColor.red;
        }
    }
};
var drawSpectrum = function (_a) {
    var imageDstBuffer = _a.imageDstBuffer, spectrum = _a.spectrum, size = _a.size, position = _a.position, rotation = _a.rotation, margin = _a.margin, color = _a.color, opacity = _a.opacity;
    var busWidth = Math.trunc(size.width / spectrum.length);
    var left = Math.trunc(position.x);
    var top = Math.trunc(position.y);
    var height = Math.trunc(size.height);
    for (var spectrumX = 0; spectrumX < spectrum.length; spectrumX++) {
        var spectrumValue = spectrum[spectrumX];
        if ((spectrumValue > 1) || (spectrumValue < 0)) {
            throw new Error('Spectrum values must be in range from 0 to 1');
        }
        var rectX = left + busWidth * spectrumX;
        var rectHeight = Math.trunc(height * spectrumValue);
        var rectY = (rotation === 'up') ? top + height - rectHeight : top;
        exports.drawRect({
            imageDstBuffer: imageDstBuffer,
            position: { x: rectX + margin, y: rectY },
            size: { width: busWidth - margin / 2, height: rectHeight },
            color: color,
            opacity: opacity,
        });
    }
};
exports.createVisualizerFrame = function (_a) {
    var backgroundImageBuffer = _a.backgroundImageBuffer, spectrum = _a.spectrum, size = _a.size, position = _a.position, rotation = _a.rotation, margin = _a.margin, color = _a.color, opacity = _a.opacity;
    var imageDstBuffer = Object.assign({}, backgroundImageBuffer);
    imageDstBuffer.data = Buffer.from(backgroundImageBuffer.data);
    var rgbSpectrumColor = (typeof color === 'string') ? hexToRgb(color) : color;
    drawSpectrum({
        imageDstBuffer: imageDstBuffer,
        spectrum: spectrum,
        size: size,
        position: position,
        rotation: rotation,
        margin: margin,
        color: rgbSpectrumColor,
        opacity: opacity,
    });
    return imageDstBuffer;
};
exports.convertToBmp = function (filePath) { return __awaiter(void 0, void 0, void 0, function () {
    return __generator(this, function (_a) {
        return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(void 0, void 0, void 0, function () {
                var image, _a;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            _b.trys.push([0, 2, , 3]);
                            return [4 /*yield*/, jimp_1.default.read(filePath)];
                        case 1:
                            image = _b.sent();
                            image.getBuffer("image/bmp", function (err, value) {
                                if (err) {
                                    reject(err);
                                }
                                resolve(value);
                            });
                            return [3 /*break*/, 3];
                        case 2:
                            _a = _b.sent();
                            throw new Error("Cannot procces image file from path: " + filePath);
                        case 3: return [2 /*return*/];
                    }
                });
            }); })];
    });
}); };
exports.parseImage = function (buffer) { return bmp_js_1.decode(buffer); };
exports.getImageColor = function (image) {
    var blueSum = 0;
    var greenSum = 0;
    var redSum = 0;
    for (var currY = 0; currY < image.height; currY++) {
        for (var currX = 0; currX < image.width; currX++) {
            var idx = (image.width * currY + currX) << 2;
            blueSum += image.data[idx + 1];
            greenSum += image.data[idx + 2];
            redSum += image.data[idx + 3];
        }
    }
    var pixelsCount = image.width * image.height;
    return {
        red: ~~(redSum / pixelsCount),
        green: ~~(greenSum / pixelsCount),
        blue: ~~(blueSum / pixelsCount)
    };
};
exports.invertColor = function (color) {
    return ({ red: 255 - color.red, green: 255 - color.green, blue: 255 - color.blue });
};
var hexToRgb = function (hex) {
    var red = parseInt(hex.substring(1, 3), 16);
    var green = parseInt(hex.substring(3, 5), 16);
    var blue = parseInt(hex.substring(5, 7), 16);
    return { red: red, green: green, blue: blue };
};
//# sourceMappingURL=image.js.map